/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Mitch Cherniack (mfc)	
//        Joon S. Lee (jl)				 	    
//  FILE: KOFFold.H 
//  PROJ: COKO, KOLA
//  DATE: 5/97	                                                    
//
// SYNOPSIS
//  The fold function former.  Given object u, 
//  function f and operator op, we have
//  
//    fold (u, f, op) ! {a1, ..., an} =
//     u op (f ! a1) op (f ! a2) ... op (f ! an)
//  
//  Note that op must be commutative and associative
//  for fold to be well-defined.
//      
//////////////////////////////////////////////////////////////////////////////

#ifndef KOLA_FFOLD_HEADER
#define KOLA_FFOLD_HEADER 

#include "KGlobals.H"
#include "KOObject.H"
#include "KOFunction.H"
#include "KONameUnion.H"
#include "KONonType.H"
#include "KOOInvoke.H"
#include "KOOPair.H"

#ifdef _OSTORE_
#include "SODA_GLOBALS.H"
#include "Iterator.H"
#include <dlfcn.h>
#endif // _OSTORE_

#include "PCState.H"
#include "KOFVar.H"
#include "KOOVar.H"
#include <string.h>

typedef NonType* (*callOptPtr)(NonType*);

class FFold : public Function
{
public:
  FFold (koObject* o, Function* f, Function* op, TFunc * ftype = NULL);
  //  Constructor.  o is an object, f the unary
  //  function and op the binary operator arguments to fold.

  int compares(NonType* nt, CState* a);
  // Checks for match between NonType and NonType.
  // Stores a variable and CIdent pair onto the stack.
  // This method is for rule matching only.
  // Rule matching is different from statement matching.
  // Duplicated variable entry is allowed here. 

  int comparesSTM(NonType* nt, CState* a);
  // Checks for match between two NonType trees (query tree and pattern tree).
  // Stores a variable and CIdent pair onto the stack.
  // This match method is used for matching statement.
  // No duplicated variable entry is allowed.
  
  NonType* transform(CState* s);
  //  Return coresponding Nontype object.
  //  CState is only used in VAR classes.
  //  No other classes need CState.
 
  void setDEPnInvDEP(char*, CState*);
  // set dependency list and inverse dependency list

  void undoSetInvDEP(char*, CState*);
  // undo changes made on inverse dependency list.
  // undo for dependency list is not necessary, since 
  // any changes on dependency list is undone by poping up
  // top frame from the environment stack.

  NonType* transformDeep(CState* s, CIEnvStackDataType* startingPt);
  // difference between transform and 
  // transformDeep is that the former uses
  // IFind method and later uses IFindDeep 
  // method. 

  void replaceChild(NonType* child, int which_child);
  // Replace a child of given tree.
  
  FFold (const FFold& other);
  //  Copy Constructor. 
  
  ~FFold ();
  //  Destructor. 

  Function *Copy () const;
  // Virtual Copy Constructor
    
  int deepeq(NonType * other);
  // Value-based equality

  NameUnion* Myname();
  // returns myname wrapped in side of a class NameUnion.

  int numChildren() const { return 3; }
  //  Number of arguments.

  koObject* One () const {return one;}
  //  What is the object that is the first argument?
  
  Function* Two () const {return two;}
  //  What is the function that is the second argument?
  
  Function* Three () const {return three;}
  //  What is the function that is the third argument?
  
  //  OResult* reduce (OResult* a);
  //  return
  //    new OResBB ("fold (" + one -> reduce () + ", " +
  //                           two -> representation () + ", " +
  //  			 three -> representation () + ")",
  //  		a)
  //
    
  char* prolog ();
  // return string representation in prolog term

  char* representation ();
  //  "FFO: FOLD (" + one -> representation () + 
  //  		  two -> representation () +
  //  		three -> representation () + ")"

  const char * Name() const { return namestring; }
  // Return the text representation of the name.

  void SetOne(NonType* a) { delete one; one = (koObject*)a; }
  // replace existing child to new one.
 
  void SetTwo(NonType* a) { delete two; two = (Function*)a; }
  // replace existing child to new one.

  void SetThree(NonType* a) { delete three; three = (Function*)a; }
  // replace existing child to new one.

  void initCurrChild() {currChild = 0;}
  // set (reset) currChild to 0.
  // This needs to be called before using nextChild() for first time.

  NonType* nextChild();
  // returns next child. (next Child = current Child + 1) 
  //If current child is the last child return NULL.

#ifdef _PROLOG_
  SP_term_ref sp_prolog ();
    // convert kola tree into prolog term and return it.
#endif // _PROLOG_ 

#ifdef _OSTORE_
  koObject* invoke(koObject* x);

  OSObject* exec(OSObject* x);
  //  Implements the fold function former.
#endif // _OSTORE_

protected:
  koObject* one;
  //  What is the object that is the first argument
  //  to the former?
  
  Function* two;
  //  What is the second (function) argument
  //  to fold?
  
  Function* three;
  //  What is the third (operator) function to fold?

  int currChild;

};

# endif
